﻿using System;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ChartView;
using System.Windows.Controls;
using System.Windows.Media.Imaging;

namespace ConsoleApplication1
{
    class Program
    {
        static void Main(string[] args)
        {
            using (FileStream stream = File.Create("chart.png"))
            {
                Task createTask = StartSTATask(() => BuildSTAChart(stream));
                createTask.Wait();
            }
        }

        [STAThread]
        private static Grid BuildSTAChart(Stream stream)
        {
            Grid host = new Grid();

            RadCartesianChart chart = BuildChart();
            host.Children.Add(chart);

            RadLegend legend = BuildChartLegend(chart);
            host.Children.Add(legend);

            PrepareElementForExport(host, 500, 300);
            Telerik.Windows.Media.Imaging.ExportExtensions.ExportToImage(host, stream, new PngBitmapEncoder());

            return host;
        }

        private static RadCartesianChart BuildChart()
        {
            RadCartesianChart chart = new RadCartesianChart { Palette = ChartPalettes.Flower };
            chart.HorizontalAxis = new CategoricalAxis();
            chart.VerticalAxis = new LinearAxis();

            CartesianSeries series = BuildBarSeries();
            ((BarSeries)series).LegendSettings = new SeriesLegendSettings { Title = "bar series 1" };
            chart.Series.Add(series);

            series = BuildBarSeries();
            ((BarSeries)series).LegendSettings = new SeriesLegendSettings { Title = "bar series 2" };
            chart.Series.Add(series);

            return chart;
        }

        private static CartesianSeries BuildBarSeries()
        {
            Random r = new Random();
            BarSeries s = new BarSeries();
            s.DataPoints.Add(new Telerik.Charting.CategoricalDataPoint { Category = "x1", Value = r.Next(0, 10), });
            s.DataPoints.Add(new Telerik.Charting.CategoricalDataPoint { Category = "x2", Value = r.Next(0, 10), });
            s.DataPoints.Add(new Telerik.Charting.CategoricalDataPoint { Category = "x3", Value = r.Next(0, 10), });
            s.DataPoints.Add(new Telerik.Charting.CategoricalDataPoint { Category = "x4", Value = r.Next(0, 10), });
            return s;
        }

        private static RadLegend BuildChartLegend(RadCartesianChart chart)
        {
            RadLegend legend = new RadLegend();
            legend.HorizontalAlignment = HorizontalAlignment.Right;
            legend.VerticalAlignment = VerticalAlignment.Top;
            legend.Items = chart.LegendItems;
            return legend;
        }

        private static void PrepareElementForExport(FrameworkElement element, double? width = null, double? height = null)
        {
            if (!element.IsInitialized)
            {
                element.BeginInit();
                element.EndInit();
            }

            double actualWidth = width ?? (element.Width > 0 ? element.Width : 400);
            double actualHeight = height ?? (element.Height > 0 ? element.Height : 300);
            element.Measure(Size.Empty);
            element.Measure(new Size(actualWidth, actualHeight));
            element.Arrange(new Rect(0, 0, actualWidth, actualHeight));
            element.UpdateLayout();
        }
        
        public static Task<T> StartSTATask<T>(Func<T> func)
        {
            var tcs = new TaskCompletionSource<T>();
            Thread thread = new Thread(() =>
            {
                try
                {
                    tcs.SetResult(func());
                }
                catch (Exception ex)
                {
                    tcs.SetException(ex);
                }
            });
            thread.SetApartmentState(ApartmentState.STA);
            thread.Start();
            return tcs.Task;
        }
    }
}
